@@ -10,6 +10,7 @@ __all__ = [
     "Version",
 ]
 
+import copy
 import datetime as dt
 import re
 import shlex
@@ -170,6 +171,12 @@ def _blank(value: Optional[_T], default: _T) -> _T:
     return value if value is not None else default
 
 
+def _equal_if_set(x: _T, y: Optional[_T], unset: Sequence[Any] = (None,)) -> bool:
+    if y in unset:
+        return True
+    return x == y
+
+
 def _detect_vcs(expected_vcs: Vcs = None) -> Vcs:
     checks = OrderedDict(
         [
@@ -374,6 +381,25 @@ class Version:
             and self.epoch == other.epoch
         )
 
+    def _matches_partial(self, other: "Version") -> bool:
+        """
+        Compare this version to another version, but ignore None values in the other version.
+        Distance is also ignored when `other.distance == 0`.
+
+        :param other: The version to compare to.
+        :return: True if this version equals the other version.
+        """
+        return (
+            _equal_if_set(self.base, other.base)
+            and _equal_if_set(self.stage, other.stage)
+            and _equal_if_set(self.revision, other.revision)
+            and _equal_if_set(self.distance, other.distance, unset=[None, 0])
+            and _equal_if_set(self.commit, other.commit)
+            and _equal_if_set(self.dirty, other.dirty)
+            and _equal_if_set(self.tagged_metadata, other.tagged_metadata)
+            and _equal_if_set(self.epoch, other.epoch)
+        )
+
     def __lt__(self, other: Any) -> bool:
         if not isinstance(other, Version):
             raise TypeError(
@@ -397,7 +423,7 @@ class Version:
         self,
         metadata: bool = None,
         dirty: bool = False,
-        format: str = None,
+        format: Union[str, Callable[["Version"], str]] = None,
         style: Style = None,
         bump: bool = False,
         tagged_metadata: bool = False,
@@ -413,8 +439,9 @@ class Version:
         :param dirty: Set this to True to include a dirty flag in the
             metadata if applicable. Inert when metadata=False.
             This is ignored when `format` is used.
-        :param format: Custom output format. You can use substitutions, such as
-            "v{base}" to get "v0.1.0". Available substitutions:
+        :param format: Custom output format. It is either a formatted string or a
+            callback. In the string you can use substitutions, such as "v{base}"
+            to get "v0.1.0". Available substitutions:
 
             * {base}
             * {stage}
@@ -438,25 +465,27 @@ class Version:
         base = self.base
         revision = self.revision
         if bump:
-            if self.stage is None:
-                base = bump_version(self.base)
-            else:
-                if self.revision is None:
-                    revision = 2
-                else:
-                    revision = self.revision + 1
+            bumped = self.bump()
+            base = bumped.base
+            revision = bumped.revision
 
         if format is not None:
-            out = format.format(
-                base=base,
-                stage=_blank(self.stage, ""),
-                revision=_blank(revision, ""),
-                distance=_blank(self.distance, ""),
-                commit=_blank(self.commit, ""),
-                tagged_metadata=_blank(self.tagged_metadata, ""),
-                dirty="dirty" if self.dirty else "clean",
-                epoch=_blank(self.epoch, ""),
-            )
+            if callable(format):
+                new_version = copy.deepcopy(self)
+                new_version.base = base
+                new_version.revision = revision
+                out = format(new_version)
+            else:
+                out = format.format(
+                    base=base,
+                    stage=_blank(self.stage, ""),
+                    revision=_blank(revision, ""),
+                    distance=_blank(self.distance, ""),
+                    commit=_blank(self.commit, ""),
+                    tagged_metadata=_blank(self.tagged_metadata, ""),
+                    dirty="dirty" if self.dirty else "clean",
+                    epoch=_blank(self.epoch, ""),
+                )
             if style is not None:
                 check_version(out, style)
             return out
@@ -525,6 +554,103 @@ class Version:
         check_version(out, style)
         return out
 
+    @classmethod
+    def parse(cls, version: str, pattern: str = _VERSION_PATTERN) -> "Version":
+        """
+        Attempt to parse a string into a Version instance.
+
+        This uses inexact heuristics, so its output may vary slightly between
+        releases. Consider this a "best effort" conversion.
+
+        :param version: Full version, such as 0.3.0a3+d7.gb6a9020.dirty.
+        :param pattern: Regular expression matched against the version.
+            Refer to `from_any_vcs` for more info.
+        """
+        try:
+            prefixed = version if version.startswith("v") else "v{}".format(version)
+            matched_pattern = _match_version_pattern(pattern, [prefixed], True)
+        except ValueError:
+            return cls(version)
+
+        base = matched_pattern.base
+        stage = matched_pattern.stage_revision
+        distance = None
+        commit = None
+        dirty = None
+        tagged_metadata = matched_pattern.tagged_metadata
+        epoch = matched_pattern.epoch
+
+        if tagged_metadata:
+            pop = []  # type: list
+            parts = tagged_metadata.split(".")
+
+            for i, value in enumerate(parts):
+                if dirty is None:
+                    if value == "dirty":
+                        dirty = True
+                        pop.append(i)
+                        continue
+                    elif value == "clean":
+                        dirty = False
+                        pop.append(i)
+                        continue
+                if distance is None:
+                    match = re.match(r"d?(\d+)", value)
+                    if match:
+                        distance = int(match.group(1))
+                        pop.append(i)
+                        continue
+                if commit is None:
+                    match = re.match(r"g?([\da-z]+)", value)
+                    if match:
+                        commit = match.group(1)
+                        pop.append(i)
+                        continue
+
+            for i in reversed(sorted(pop)):
+                parts.pop(i)
+
+            tagged_metadata = ".".join(parts)
+
+        if distance is None:
+            distance = 0
+        if tagged_metadata is not None and tagged_metadata.strip() == "":
+            tagged_metadata = None
+
+        return cls(
+            base,
+            stage=stage,
+            distance=distance,
+            commit=commit,
+            dirty=dirty,
+            tagged_metadata=tagged_metadata,
+            epoch=epoch,
+        )
+
+    def bump(self, index: int = -1) -> "Version":
+        """
+        Increment the version.
+
+        The base is bumped unless there is a stage defined, in which case,
+        the revision is bumped instead.
+
+        :param index: Numerical position to increment in the base. Default: -1.
+            This follows Python indexing rules, so positive numbers start from
+            the left side and count up from 0, while negative numbers start from
+            the right side and count down from -1.
+            Only has an effect when the base is bumped.
+        :return: Bumped version.
+        """
+        bumped = copy.deepcopy(self)
+        if bumped.stage is None:
+            bumped.base = bump_version(bumped.base, index)
+        else:
+            if bumped.revision is None:
+                bumped.revision = 2
+            else:
+                bumped.revision = bumped.revision + 1
+        return bumped
+
     @classmethod
     def from_git(cls, pattern: str = _VERSION_PATTERN, latest_tag: bool = False) -> "Version":
         r"""
@@ -993,6 +1119,8 @@ def get_version(
     first_choice: Callable[[], Optional[Version]] = None,
     third_choice: Callable[[], Optional[Version]] = None,
     fallback: Version = Version("0.0.0"),
+    ignore: Sequence[Version] = None,
+    parser: Callable[[str], Version] = Version,
 ) -> Version:
     """
     Check pkg_resources info or a fallback function to determine the version.
@@ -1005,10 +1133,20 @@ def get_version(
     :param third_choice: Callback to determine a version if the installed
         package cannot be found by name.
     :param fallback: If no other matches found, use this version.
+    :param ignore: Ignore a found version if it is part of this list. When
+        comparing the found version to an ignored one, fields with None in the ignored
+        version are not taken into account. If the ignored version has distance=0,
+        then that field is also ignored.
+    :param parser: Callback to convert a string into a Version instance.
+        This will be used for the second choice.
+        For example, you can pass `Version.parse` here.
     """
+    if ignore is None:
+        ignore = []
+
     if first_choice:
         first_ver = first_choice()
-        if first_ver:
+        if first_ver and not any(first_ver._matches_partial(v) for v in ignore):
             return first_ver
 
     try:
@@ -1016,13 +1154,15 @@ def get_version(
     except ImportError:
         import importlib_metadata as ilm  # type: ignore
     try:
-        return Version(ilm.version(name))
+        ilm_version = parser(ilm.version(name))
+        if not any(ilm_version._matches_partial(v) for v in ignore):
+            return ilm_version
     except ilm.PackageNotFoundError:
         pass
 
     if third_choice:
         third_ver = third_choice()
-        if third_ver:
+        if third_ver and not any(third_ver._matches_partial(v) for v in ignore):
             return third_ver
 
     return fallback
